import argparse as arg
import numpy

class Tree:
    def __init__(self, heigh):
        self.height = height
        # initialize a base tree 
        self.Root = self.newLevel(height - 1)

    def treeToStr(self):
        self.leafCounter = 0
        self.nodeCounter = 0.0000001
        return self.getNewick(self.height)

    def getNewick(self, height):
        if height > 0:
            string = '(' + self.getNewick(height - 1) + ':1,'
            string = string + self.getNewick(height - 1) + ':1)' + "%.7f" % self.nodeCounter
            self.nodeCounter += 0.0000001
        else:
            string = str(self.leafCounter)
            self.leafCounter += 1
        return string


    def generateFreqVect(self):
        self.x = []
        self.recursiveSampling(self.height, self.Root, 1)
        vector = ""
        for freq in self.x:
            vector += str(freq) + " "
        return vector 
        
    def recursiveSampling(self, height, node, parentVal):
        if height > 0:
            nodeVal = parentVal * node[0]
            if height > 1: 
                self.recursiveSampling(height - 1, node[1][0], nodeVal)
                self.recursiveSampling(height - 1, node[1][1], parentVal - nodeVal)
            else:
                self.recursiveSampling(height - 1, [], nodeVal)
                self.recursiveSampling(height - 1, [], parentVal - nodeVal)
        else:
            self.x.append(parentVal)


    def sampleTree(self, baseTree, var):       
        self.var = var  
        # sample the paramaters of the class tree from the base tree
        self.sampleNode(self.Root, baseTree.Root, self.height)
        

    def sampleNode(self, node, baseNode, height):
        dist = 1 - float(height) / self.height
        var = dist * self.var
        if var > 0:
            node[0] = min(1,max(0,numpy.random.normal(baseNode[0], var, 1)[0]))
        else:
            node[0] = baseNode[0]
        if node[1] != -1 and baseNode[1] != -1:
            self.sampleNode(node[1][0], baseNode[1][0], height - 1)
            self.sampleNode(node[1][1], baseNode[1][1], height - 1)

    def newLevel(self, hight):
        if hight > 0:
            node = [[],[]]
            node[0] = self.newLevel(hight - 1)
            node[1] = self.newLevel(hight - 1)
        else:
            node = -1
        return [numpy.random.random(), node]


if __name__=='__main__':
    opt=arg.ArgumentParser(description='Simulate OTU frequency vectors that take into account simulated phylogeny.')
    opt.add_argument('-c', default = 2, type = int, help='Number of classes. Default: 2.')
    opt.add_argument('-n', default = 20, type = int, help='Number of samples per class. Default: 20.')
    opt.add_argument('-v', type = float, default = 1, help='Within-class variance. Default: 1.')
    opt.add_argument('-w', type = float, default = 1, help='Between-class variance. Default: 1.')
    opt.add_argument('-t', default = 5, type = int,  help='Tree height. Default: 5.')
    opt.add_argument('-o', default = 'out', type = str, help='Output prefix. Default: "out". Three files are generated: [output prefix].tre - phylogenetic tree in a Newick format, [output prefix]_data.txt - OTU frequency vectors (samples), [output prefix]_labels.txt - class labels of the samples.')
    args=opt.parse_args()
    nClasses = args.c
    nClassSamples = args.n
    varIn = args.v
    varOut = args.V
    height = args.t

    # generate a base tree    
    baseTree = Tree(height)
    # print tree
    fout = open(args.o + ".tre", "w")
    fout.write(baseTree.treeToStr()) 
    fout.close()
     
    # generate a tree for each class
    fout = open(args.o + ".x", "w") 
    counter = 1
    for j in range(nClasses):
        classTree = Tree(height)
        classTree.sampleTree(baseTree,varOut)
        
        # generate samples for each class:
        for i in range(nClassSamples):
            sampleTree = Tree(height)
            sampleTree.sampleTree(classTree,varIn) 
            # generate OTU frequency vector and write it to file
            vect = sampleTree.generateFreqVect()
            fout.write(str(counter) + " " + vect)
            if i < nClassSamples - 1 or j < nClasses - 1:
                fout.write("\n")
            counter += 1
    fout.close()
    # write class assignments to file
    fout = open(args.o + ".y", "w")
    counter = 1
    for i in range(nClasses):
        fout.write("> " + str(i+1) + "\n")
        for j in range(nClassSamples):
            fout.write(str(counter) + " ")
            counter += 1
        if i < nClasses - 1:
            fout.write("\n")
    fout.close()
